"""
Standards:

Internally, positions are a list, where index 0 is X and 1 is Y.
A-H are 1-8 respectively.
These are changed to algebraic notation when displayed to the user.
Example: [0,0]  = 'a1', [7,6]  = 'h7'

"""

##Either R-S, R-R, U-U, U-R, U-S
##R = random, S = stockfish, U = user
run = "U-U"

import os
import random

from stockfish import Stockfish
import board as boardM
from board import *
from chessman import *
from enums import *

os.chdir(__file__+"/../")

def main():
    board = Board()
    board.to_image().save("Boards/0.0.png")

    # stockfish = Stockfish(path="stockfish_15.1_win_x64_popcnt/stockfish-windows-2022-x86-64-modern.exe",depth=10,parameters={"Threads": 2,"Minimum Thinking Time":1,"Hash":2048})
    # stockfish.set_fen_position("rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w - - 0 0")
    # for idx in range(1,100):
    #     bm = stockfish.get_best_move()

    #     st = convert_move(bm[:2],MoveNotation.internal)
    #     dt = convert_move(bm[2:],MoveNotation.internal)
    #     squares = board.squares
    #     piece:chessman.Chessman = squares[st[0]][st[1]].chessman
    #     try:
    #         piece.move(dt)
    #     except AttributeError:
    #         raise AttributeError(bm)

    #     stockfish.make_moves_from_current_position([bm])

    #     board.to_image(str(idx/2))

    def isValidMove(start:list[int,int],dest:list[int,int],board:Board,colour:Colour) -> bool:
        try:
            piece = board.squares[start[0]][start[1]].chessman
        except IndexError:
            return False
        if piece:
            if piece.colour == colour:
                if dest in piece.getavailablemoves():
                    return True

    if run == "U-U":
        idx = 0
        oppColour = Colour.black
        while True:
            idx += 1

            anyMoves = False
            for i in board.pieces[1 if ~oppColour == Colour.white else 0]:
                moves = i.getavailablemoves()
                if len(moves) > 0:
                    anyMoves = True
                    print(i.type)
                    convMoves = [boardM.convert_move((i.letter,i.position,mov),MoveNotation.longalgebraic) for mov in moves]
                    print(convMoves)
                    print()

            if not anyMoves:
                print("\n\n")
                if board.incheck:
                    print("Game ended by Checkmate!",f"{oppColour} wins.")
                    print("\n\n")
                    return GameResolution.victory,oppColour
                else:
                    print("Game ended by Stalemate!")
                    print("\n\n")
                    return GameResolution.draw,DrawCondition.stalemate

            move = input("Move: ")
            st = convert_move(move[:2],MoveNotation.internal)
            dt = convert_move(move[2:],MoveNotation.internal)
            if isValidMove(st,dt,board,~oppColour):
                oppColour = ~oppColour
                board.squares[st[0]][st[1]].chessman.move(dt)
                board.to_image().save(f"Boards/{str(idx)}.png")
                print(board.to_text(False))
            else:
                print("Invalid move!")

    if run == "R-R":
    
        for idx in range(1,100):
            pieces = []
            for i in board.squares:
                for j in i:
                    if getattr(j.chessman,"colour",None) == Colour.white and idx % 2 == 0:
                        pieces.append(j.chessman)
                    elif getattr(j.chessman,"colour",None) == Colour.black and idx % 2 == 1:
                        pieces.append(j.chessman)

            found = False
            for i in range(1,100):
                random.shuffle(pieces)
                piece = pieces[0]
                moves = piece.getavailablemoves()
                if len(moves) >= 1:
                    found = True
                    break
            
            if found:  
                random.shuffle(moves)
                piece.move(moves[0])
                #print(board.to_text(True))
                board.to_image().save(f"Boards/{str(idx/2)}.png")
            else:
                print("Failed to find a valid move, game aborted.")
                return -1

        print("Simulation finished!")

if __name__ == "__main__":
    main()



from enum import Enum

__all__ = (
    "ChessmanType",
    "ChessmanState",
    "GameResolution",
    "MoveNotation",
    "VictoryCondition",
    "DrawCondition",
    "Rulesets",
    "Colour",
)

class ChessmanType(Enum):
    """Denotes a Chessman type.

    Values are 40-49, where 49 is none.
    """
    pawn = 40
    knight = 41
    bishop = 42
    rook = 43
    queen = 44
    king = 45
    none = 49

    def __str__(self) -> str:
        return self.name
    
    def __int__(self) -> int:
        return self.value
    
class ChessmanState(Enum):
    """Denotes a State for a Chessman, either Alive or Dead.

    Dead should not generally be used.

    Values are 50-59.
    """

    dead = 50
    alive = 51
    captured = 52

class Colour(Enum):
    """Denotes a Colour.
    Does not denote an RGB value for these.

    Values are 10-19.
    """
    black = 10
    white = 11

    def __str__(self) -> str:
        return self.name
    
    def __invert__(self):
        if self.value == 10:
            return Colour.white
        else:
            return Colour.black

class Rulesets(Enum):
    """Denotes a Ruleset for Skek-Chess.

    Standard - the standard rules of Chess, as clarified by FIDE (https://www.fide.com/FIDE/handbook/LawsOfChess.pdf).

    Values are 0-9.
    """
    standard = 0

    def __str__(self) -> str:
        return self.name
    

class GameResolution(Enum):
    """Denotes the basic game ending, either victory for either party or a draw.

    Values are 20-21.
    """
    victory = 20
    draw = 21

    def __str__(self) -> str:
        return self.name

class VictoryCondition(Enum):
    """Denotes the Condition that resulted in a Victory for either party.

    A Timeout can be either a Victory or a Draw depending on the game state.
    https://www.chess.com/article/view/how-chess-games-can-end-8-ways-explained Should explain these.

    Values are 22-23.
    """
    checkmate = 22
    timeout = 23

    def __str__(self) -> str:
        return self.name

class DrawCondition(Enum):
    """Denotes the Condition that resulted in a Draw.

    A Timeout can be either a Victory or a Draw depending on the game state.
    https://www.chess.com/article/view/how-chess-games-can-end-8-ways-explained Should explain these.
    Values are 24-29.

    """
    stalemate = 24
    material = 25
    fiftymoverule = 26
    repetition = 27
    agreement = 28
    timeout = 29

    def __str__(self) -> str:
        return self.name
    
class PlayerType(Enum):
    """Denotes a Type of Player that the user is playing against.

    Engine refers to any Engine run in Skek-Chess.
    Other refers to any Other Player Type, such as an External Engine or AI.

    Values are 60-69.
    """
    human = 60
    engine = 61
    other = 69

    def __str__(self) -> str:
        return self.name
    
class MoveNotation(Enum):
    """Denotes a move notation.

    LongAlgebraic uses the starting position followed by the destination position. A piece letter is appended if promoting.
    Algebraic - https://en.wikipedia.org/wiki/Algebraic_notation_(chess)
    Internal is a list of 2 base 10 integers starting from 0.

    Values are 70-75.
    """

    longalgebraic = 70
    algebraic = 71
    internal = 75



from copy import deepcopy
import os

from enums import *
import board

os.chdir(__file__+"/../")
__all__ = (
    "Chessman",
)

class Chessman:
    """Base class for a Chessman (Piece, Pawn, King).
    Movement rules and traits defined in sub-classes.

    Properties
    -----------
    type: :class:`ChessmanType`
        What Chessman is this?
    colour: :class:`Colour`
        Who does this Chessman belong to?
    king: :class:`bool`
        Is this Chessman type a King (will lose if this Chessman is checkmated)?
    
    state: :class:`ChessmanState`
        Alive or Captured?
    position: :class:`List[int,int]`
        Where on the board is this Chessman?

    canmove: :class:`bool`
        Can this Chessman move, or is it blocked or pinned to the King?
    hasmoved: :class:`bool`
        Has this Chessman moved already? Used for castling rights and pawns.
    
    availablemovement: :class:`List[List[int,int]]`
        Where can this Chessman move to? 'a_castle' and 'h_castle' used to denote castling, respectively.
    controlledsquares: :class:`List[List[int,int]]
        Which squares is this Chessman controlling?

    availablepromotions: :class:`List[Chessman]`
        Who can this Chessman promote to? Empty until can promote, i.e. Pawn reaching final Rank.
    promotedfrom: :class:`List[Chessman]`
        Who was this Chessman before promoting?
    

    """

    king = False
    letter = "-"
    type = ChessmanType.none

    def checkforcheck(self,possibleMoves:list[list[int,int]]):
        "Returns a list of possible moves that would not put the King in check. Helper function for getavailablemoves()."
        b:board.Board = self.board

        if not self.king:
            kingPos = None
            for piece in b.pieces[1 if self.colour == Colour.white else 0]:
                if piece.type == ChessmanType.king:
                    kingPos = piece.position
                    break
        
        originalMoves = deepcopy(possibleMoves)
        for move in originalMoves:
            newBoard = deepcopy(b)
            curSquare = newBoard.squares[self.position[0]][self.position[1]]
            newChessman = curSquare.chessman
            if not newChessman:
                continue
            curSquare.chessman = None

            newSquare = newBoard.squares[move[0]][move[1]]
            del(newSquare.chessman)
            newChessman.square = newSquare
            newSquare.chessman = newChessman

            if self.king:
                kingPos = move
            
            for enemy in newBoard.pieces[1 if self.colour == Colour.black else 0]:
                enemy:Chessman
                if kingPos in enemy.getattackedpositions(b=newBoard,skipCheckCheck=True):
                    possibleMoves.remove(move)
                    break
            del(newBoard)

        return possibleMoves

    def getavailablemoves(self,b=None,skipCheckCheck=False) -> list[list[int,int]]:
        "Returns a list of available moves."
        return []
    
    def getattackedpositions(self,b=None,skipCheckCheck=False) -> list[list[int,int]]:
        "Returns positions that are being attacked by this Chessman. Identical to getavailablemoves for all pieces but not pawns."
        return self.getavailablemoves(b=b,skipCheckCheck=skipCheckCheck)
    
    def capture(self,square):
        "Capture a Chessman on a Square."
        square.chessman.state = ChessmanState.captured
        del(square.chessman)

    def move(self,pos:list[int]):
        "Move to a square, and capture if needed."
        self.hasmoved = True
        self.square.chessman = None
        newSquare = self.board.squares[pos[0]][pos[1]]
        if newSquare.chessman:
            self.capture(newSquare)
        newSquare.chessman = self
        self.square = newSquare
        self.position = pos

        kingPos = None
        for piece in self.board.pieces[1 if self.colour == Colour.black else 0]:
            if piece.king:
                kingPos = piece.position
                break

        for move in self.getavailablemoves(skipCheckCheck=True):
            if move == kingPos:
                self.board.incheck = True
                break
        else:
            self.board.incheck = False

    def __init__(self,colour:Colour,b=None,square=None,pos=[0,0]):
        self.board = b
        self.square = square
        self.colour = colour

        self.state = ChessmanState.alive
        self.position = pos

        self.incheck = False
        self.canmove = True
        self.hasmoved = False

        self.availablemovement = []
        self.controlledsquares = []

        self.availablepromotions = []
        self.promotedfrom = []


    def __str__(self):
        return f"Type: {self.type}\nColour: {self.colour}\nState: {self.state}\nPosition: {self.position}\nCanMove: {self.canmove}\nHasMoved: {self.hasmoved}\nAvailablePromotions: {str(self.availablepromotions)}\nPromotedFrom: {str(self.promotedfrom)}"

class Pawn(Chessman):
    "The Pawn."

    king = False
    type = ChessmanType.pawn

    letter = "p"
    unicode = ["♟︎","♙"]

    def getattackedpositions(self,b=None,skipCheckCheck=False) -> list[list[int]]:
        b:board.Board = b if b else self.board
        sq = b.squares

        pos = self.position
        col = self.colour

        yDir = 1 if col == Colour.white else -1
        moves = []
        try:
            Cman = sq[pos[0]+1][pos[1]+yDir].chessman
            if Cman != None and pos[0] < 7 and pos[1]+yDir <= 7 and pos[1]+yDir >= 0:
                if Cman.colour != col:
                    moves.append([pos[0]+1,pos[1]+yDir])
        except IndexError:
            pass
            ##Apparantely negative index underflows are fine but positive overflows are not. I retract my statement about python being a nice language

        try:
            Cman = sq[pos[0]-1][pos[1]+yDir].chessman
            if Cman != None and pos[0] > 0 and pos[1]+yDir <= 7 and pos[1]+yDir >= 0:
                if Cman.colour != col:
                    moves.append([pos[0]-1,pos[1]+yDir])
        except IndexError:
            pass

        if not skipCheckCheck:
            moves = self.checkforcheck(moves)

        return moves


    def getavailablemoves(self,b=None,skipCheckCheck=False) -> list[list[int]]:
        ## Fake boards are used to determine check or not
        b:board.Board = b if b else self.board
        sq = b.squares

        pos = self.position
        mov = self.hasmoved
        col = self.colour

        moves = self.getattackedpositions(b=b,skipCheckCheck=skipCheckCheck)

        yDir = 1 if col == Colour.white else -1
        forwardBlocked = False
        try:
            if sq[pos[0]][pos[1]+yDir].chessman == None and pos[1]+yDir <= 7 and pos[1]+yDir >= 0:
                moves.append([pos[0],pos[1]+yDir])
            else:
                forwardBlocked = True
        except IndexError:
            pass

        if not mov and not forwardBlocked:
            try:
                if sq[pos[0]][pos[1]+(yDir*2)].chessman == None and pos[1]+(yDir*2) <= 7 and pos[1]+(yDir*2) >= 0:
                    moves.append([pos[0],pos[1]+(yDir*2)])
            except IndexError:
                pass

        if not skipCheckCheck:
            moves = self.checkforcheck(moves)

        return moves
    
class Knight(Chessman):
    "The Knight Chess Piece."

    king = False
    type = ChessmanType.knight

    letter = "n"
    unicode = ["♞","♘"]

    def getavailablemoves(self,b=None,skipCheckCheck=False) -> list[list[int]]:
        ## Fake boards are used to determine check or not
        b:board.Board = b if b else self.board
        sq = b.squares

        pos = self.position
        col = self.colour

        moves = []

        for x,l in enumerate(sq):
            for y,square in enumerate(l):
                if square.distance(pos) > 2:
                    continue
                if (abs(x-pos[0]) == 2 and abs(y-pos[1]) == 1) or (abs(x-pos[0]) == 1 and abs(y-pos[1]) == 2):
                    if not square.chessman:
                        moves.append(square.position)
                    elif square.chessman.colour != col:
                        moves.append(square.position)

        if not skipCheckCheck:
            moves = self.checkforcheck(moves)

        return moves
    
def rookMoves(self,b=None,skipCheckCheck=False) -> list[list[int]]:
    "Returns available moves for Rook-like pieces."

    b:board.Board = b if b else self.board
    sq = b.squares

    pos = self.position
    mov = self.hasmoved
    col = self.colour

    moves = []

    for y in range(pos[1]+1,8):
        square = sq[pos[0]][y]
        if square.chessman == None:
            moves.append(square.position)
        else:
            if square.chessman.colour != col:
                moves.append(square.position)
            break

    for y in range(pos[1]-1,-1,-1):
        square = sq[pos[0]][y]
        if square.chessman == None:
            moves.append(square.position)
        else:
            if square.chessman.colour != col:
                moves.append(square.position)
            break

    for x in range(pos[0]+1,8):
        square = sq[x][pos[1]]
        if square.chessman == None:
            moves.append(square.position)
        else:
            if square.chessman.colour != col:
                moves.append(square.position)
            break

    for x in range(pos[0]-1,-1,-1):
        square = sq[x][pos[1]]
        if square.chessman == None:
            moves.append(square.position)
        else:
            if square.chessman.colour != col:
                moves.append(square.position)
            break

    if not skipCheckCheck:
        moves = self.checkforcheck(moves)
    
    return moves

def bishopMoves(self,b=None,skipCheckCheck=False):
    "Returns available moves for Bishop-like pieces."

    b:board.Board = b if b else self.board
    sq = b.squares

    pos = self.position
    col = self.colour

    moves = []

    for i in range(1,8):
        if pos[0] + i < 8 and pos[1] + i < 8:
            square = sq[pos[0]+i][pos[1]+i]
            if square.chessman == None:
                moves.append(square.position)
            else:
                if square.chessman.colour != col:
                    moves.append(square.position)
                break
        else:
            break

    for i in range(1,8):
        if pos[0] - i > -1 and pos[1] + i < 8:
            square = sq[pos[0]-i][pos[1]+i]
            if square.chessman == None:
                moves.append(square.position)
            else:
                if square.chessman.colour != col:
                    moves.append(square.position)
                break
        else:
            break

    for i in range(1,8):
        if pos[0] - i > -1 and pos[1] - i > -1:
            square = sq[pos[0]-i][pos[1]-i]
            if square.chessman == None:
                moves.append(square.position)
            else:
                if square.chessman.colour != col:
                    moves.append(square.position)
                break
        else:
            break

    for i in range(1,8):
        if pos[0] + i < 8 and pos[1] - i > -1:
            square = sq[pos[0]+i][pos[1]-i]
            if square.chessman == None:
                moves.append(square.position)
            else:
                if square.chessman.colour != col:
                    moves.append(square.position)
                break
        else:
            break

    if not skipCheckCheck:
        moves = self.checkforcheck(moves)
    return moves

def kingMoves(self,b=None,skipCheckCheck=False):
    "Returns available moves for King-like pieces."

    b:board.Board = b if b else self.board
    sq = b.squares

    pos = self.position
    mov = self.hasmoved
    col = self.colour

    moves = []

    idx = 0
    for l in sq:
        for square in l:
            if idx >= 8:
                break
            if square.distance(pos) == 1:
                if getattr(square.chessman,"colour",None) != col:
                    moves.append(square.position)
                idx += 1
                
    for piece in b.pieces[1 if col == Colour.white else 0]:
        if piece.type == ChessmanType.rook and not piece.hasmoved:
            ##castle
            pass

    if not skipCheckCheck:
        moves = self.checkforcheck(moves)

    return moves

def castleMoves(self:Chessman):
    "Returns available castling moves from this piece. Returns an empty list if none are available, either due to temporary or permenant reasons."

    b:board.Board = self.board
    sq = b.squares

    pos = self.position
    mov = self.hasmoved
    col = self.colour

    moves = []

    if not mov:
        king = self if self.type == ChessmanType.king else None

        idx = 0
        rooks = []
        for p in b.pieces:
            if p.type == ChessmanType.king and not king:
                idx += 1
                king = p
            elif p.type == ChessmanType.rook and not p.hasmoved:
                idx += 1
                rooks.append(p)

        if king.hasmoved or len(rooks) <= 0:
            return []
        else:
            pass
            
        

    return moves
    
class Rook(Chessman):
    "The Rook Chess Piece."

    king = False
    type = ChessmanType.rook

    letter = "r"
    unicode = ["♜","♖"]

    def getavailablemoves(self,b=None,skipCheckCheck=False):
        return rookMoves(self,b=b,skipCheckCheck=skipCheckCheck)
    
class Bishop(Chessman):
    "The Bishop Chess Piece."

    king = False
    type = ChessmanType.bishop

    letter = "b"
    unicode = ["♝","♗"]

    def getavailablemoves(self,b=None,skipCheckCheck=False) -> list[list[int]]:
        return bishopMoves(self,b=b,skipCheckCheck=skipCheckCheck)
    
class Queen(Chessman):
    "The Queen Chess Piece."

    king = False
    type = ChessmanType.queen

    letter = "q"
    unicode = ["♛","♕"]

    def getavailablemoves(self,b=None,skipCheckCheck=False):
        return rookMoves(self,b=b,skipCheckCheck=skipCheckCheck) + bishopMoves(self,b=b,skipCheckCheck=skipCheckCheck)

class King(Chessman):
    "The King."

    king = True
    type = ChessmanType.king

    letter = "k"
    unicode = ["♚","♔"]

    def getavailablemoves(self,b=None,skipCheckCheck=False) -> list[list[int]]:
        return kingMoves(self,b=b,skipCheckCheck=skipCheckCheck)


from copy import deepcopy
import os
from PIL import Image, ImageDraw

import chessman
from enums import *

os.chdir(__file__+"/../")

# default_layout = """
# 8 | r n b q k b n r
# 7 | p p p p p p p p
# 6 | . . . . . . . .
# 5 | . . . . . . . .
# 4 | . . . . . . . .
# 3 | . . . . . . . .
# 2 | P P P P P P P P
# 1 | R N B Q K B N R
#     ― ― ― ― ― ― ― ―
#     A B C D E F G H
# """

default_layout = {
    0: [chessman.Rook(Colour.white),chessman.Knight(Colour.white),chessman.Bishop(Colour.white),chessman.Queen(Colour.white),
        chessman.King(Colour.white),chessman.Bishop(Colour.white),chessman.Knight(Colour.white),chessman.Rook(Colour.white)],
    1: [chessman.Pawn(Colour.white) for _ in range(0,8)],
    6: [chessman.Pawn(Colour.black) for _ in range(0,8)],
    7: [chessman.Rook(Colour.black),chessman.Knight(Colour.black),chessman.Bishop(Colour.black),chessman.Queen(Colour.black),
        chessman.King(Colour.black),chessman.Bishop(Colour.black),chessman.Knight(Colour.black),chessman.Rook(Colour.black)],
}

# __all__ = (
#     "Square",
#     "Board",
# )

class Square:
    """Represents a Square on a Board.

    Properties
    -----------
    chessman: :class:`Chessman`
        A chessman instance to place on this Square. Can be None.

    position: :class:`List[int]`
        The position of this Square.

    Methods
    -----------
    distance(position :class:`List[int]`):
        Returns the integer diagonal distance between two squares.
    """

    def __init__(self,position:list[int],chessman:chessman.Chessman=None):
        self.position = position
        self.chessman = chessman
        self.colour = Colour.black if (position[0]+position[1])%2 == 1 else Colour.white

    def __str__(self):
        self.chessman = getattr(self,"chessman","")
        return f"Position: {self.position[0]},{self.position[1]}\nChessman: {self.chessman}\nColour: {self.colour}"


    def distance(self,p:list[int]):
        "Returns the integer diagonal distance between two squares."
        p1 = self.position

        distX = abs(p[0]-p1[0])
        distY = abs(p[1]-p1[1])
        diagCut = min(distX,distY)

        return distX + distY - diagCut
    
    def isatdiagonal(self,pos:list[int]):
        "Returns whether this square is at a diagonal to another square."
        sPos = self.position
        if abs(sPos[0] - pos[0]) == (sPos[1] - pos[1]):
            return True
        
class Board:
    """Represents a chess Board.
    """
    
    def __init__(self,ruleset=Rulesets.standard):
        
        squares = [[Square([j,i]) for i in range(0,8)] for j in range(0,8)] ##I love that this works, python is such a nice language
        pieces = [[],[]]
        for x,l in enumerate(squares,start=0):
            for y,square in enumerate(l,start=0):
                if y not in [0,1,6,7]:
                    continue
                Cman = deepcopy(default_layout[y][x])
                Cman.board = self
                Cman.square = square
                Cman.position = [x,y]
                pieces[1 if Cman.colour == Colour.white else 0].append(Cman)
                square.chessman = Cman

        self.pieces = pieces
        self.squares = squares
        self.incheck = False

    def to_text(self,unicode:bool):
        text = ""
        for x in reversed(self.squares):
            for square in x:
                Cman = square.chessman
                if Cman:
                    if Cman.colour == Colour.black:
                        text += " "+ (Cman.letter.lower() if not unicode else Cman.unicode[0])
                    else:
                        text += " "+ (Cman.letter.upper() if not unicode else Cman.unicode[1])
                else:
                    text += " ."
            text += "\n"
        return text
    

    def to_image(self):
        background = Image.open(__file__+"/../"+"Images/board.png").convert("RGBA")
        for x,l in enumerate(self.squares):
            for y,square in enumerate(l):
                if square.chessman:
                    img = Image.open(__file__+"/../"+"Images/"+str("b" if square.chessman.colour == Colour.black else "w")+str(square.chessman.letter)+".png").convert("RGBA")
                    background.paste(img,(x*150,1050-(y*150)),img)
        return background
    

letters_numbers = {
    "a":0,
    "b":1,
    "c":2,
    "d":3,
    "e":4,
    "f":5,
    "g":6,
    "h":7
}
    

def baseN(num,b,numerals="0123456789abcdefghijklmnopqrstuvwxyz"):
    return ((num == 0) and numerals[0]) or (baseN(num // b, b, numerals).lstrip(numerals[0]) + numerals[num % b])
    

def convert_coordinate(position:list[int,int]|str) -> list[int,int]|str:
    "Converts a coordinate to the alternative format."
    if type(position) == str:
        pass
    elif type(position) == list:
        return f"{baseN(position[0]+10,18)}{position[1]+1}"
    

def convert_move(move:str|tuple[str,list[int,int],list[int,int]],newFormat:MoveNotation) -> str:
    """Converts a move to another notation.
    To convert to Long Algebraic, move must be a tuple of the Chessman's letter, the Chessman's starting position and the destination.
    """
    if type(move) == tuple and len(move) == 3:
        if newFormat == MoveNotation.longalgebraic:
            _,start,dest = move
            return convert_coordinate(start)+convert_coordinate(dest)
    elif type(move) == str and len(move) == 2:
        if newFormat == MoveNotation.internal:
            return [letters_numbers[move[0]],int(move[1])-1]
